//
//  FPWCSApi2WebRTCMediaProvider.h
//  FPWCSApi
//
//  Created by user on 17/10/2016.
//  Copyright © 2016 user. All rights reserved.
//
#import <os/log.h>

#import <WebRTC/RTCPeerConnectionFactory.h>
#import <WebRTC/RTCPeerConnection.h>
#import <WebRTC/RTCVideoRenderer.h>
#import <WebRTC/RTCSessionDescription.h>
#import <WebRTC/RTCMediaConstraints.h>
#import <WebRTC/RTCVideoCapturer.h>
#import <WebRTC/RTCCameraVideoCapturer.h>
#import <AVFoundation/AVCaptureSession.h>
#import <WebRTC/RTCVideoTrack.h>
#import "FPWCSApi2Model+internal.h"
#import "FPWCSCameraVideoCapturer.h"

@class FPWCSApi2MediaConnection;
@class FPWCSApi2MediaResource;

@protocol FPWCSApi2MediaConnectionDelegate <NSObject>

- (void)connection:(FPWCSApi2MediaConnection *)connection
    didCreateOffer:(NSString *)sdp;
- (void)connection:(FPWCSApi2MediaConnection *)connection
    didFailToCreateDescription:(NSError *)error;
- (void)connection:(FPWCSApi2MediaConnection *)connection
didFailToSetDescription:(NSError *)error;

@end

@interface FPWCSApi2WebRTCMediaProvider : NSObject

@property RTCPeerConnectionFactory *factory;
@property os_log_t logger;

- (void)getMediaAccess:(FPWCSApi2MediaConstraints *)constraints display:(id<RTCVideoRenderer>)display;

- (void) captureWithDeviceForPosition:(AVCaptureDevicePosition)position withConstraints:(FPWCSApi2VideoConstraints *) constraints withCapturer:(RTCCameraVideoCapturer *) capturer;

- (FPWCSApi2MediaDeviceList *)getMediaDevices;

- (NSArray *)getSupportedVideoResolutions;

- (BOOL)releaseMedia:(id<RTCVideoRenderer>)display;

- (FPWCSApi2MediaResource *)getMediaResource:(id<RTCVideoRenderer>)display;

- (FPWCSApi2MediaConnection *)createConnection:(NSString *)connectionID
                                withConfig:(RTCConfiguration *) config
                                withDelegate:(id<FPWCSApi2MediaConnectionDelegate>)delegate
                                withLocalDisplay:(id<RTCVideoRenderer>)localDisplay withRemoteDisplay:(id<RTCVideoRenderer>) remoteDisplay;

@end

@interface FPWCSApi2MediaConnection : NSObject<RTCPeerConnectionDelegate>

@property os_log_t logger;

@property RTCPeerConnection *connection;
@property id<FPWCSApi2MediaConnectionDelegate> delegate;
@property BOOL closed;

- (instancetype)initWithId:(NSString *)connectionID
            initWithConfig:(RTCConfiguration *) config
            initWithDelegate:(id<FPWCSApi2MediaConnectionDelegate>)delegate
            initWithFactory:(RTCPeerConnectionFactory *)factory
            initWithSource:(FPWCSApi2MediaResource *)localMediaResource initWithRemoteDisplay:(id<RTCVideoRenderer>) remoteVideoDisplay;

- (void)setRemoteSdp:(RTCSessionDescription *)sdp;

- (void)createOffer:(BOOL)receiveAudio sendAudio:(BOOL)sendAudio receiveVideo:(BOOL)receiveVideo sendVideo:(BOOL)sendVideo;

- (void)createAnswer:(BOOL)receiveAudio sendAudio:(BOOL)sendAudio receiveVideo:(BOOL)receiveVideo sendVideo:(BOOL)sendVideo;

- (void)close;

- (void)muteAudio;

- (void)unmuteAudio;

- (void)muteVideo;

- (void)unmuteVideo;

- (void)lockCameraOrientation;

- (void)unlockCameraOrientation;

- (BOOL)isAudioMuted;

- (BOOL)isVideoMuted;

@end

@interface FPWCSApi2MediaResource : NSObject

@property RTCMediaStream *stream;

@property id<RTCVideoRenderer> view;

@property FPWCSApi2MediaConstraints *constraints;

@property FPWCSCameraVideoCapturer *cameraVideoCapturer;

@property AVCaptureDevicePosition cameraPosition;

- (void)close;

@end
